@@ -8,7 +8,6 @@ from pytype import datatypes
 from pytype import utils
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
-from pytype.pytd import visitors
 
 log = logging.getLogger(__name__)
 
@@ -723,10 +722,7 @@ class PyTDSignature(utils.VirtualMachineWeakrefMixin):
     except self.vm.convert.TypeParameterError:
       # The return type contains a type parameter without a substitution.
       subst = subst.copy()
-      visitor = visitors.CollectTypeParameters()
-      return_type.Visit(visitor)
-
-      for t in visitor.params:
+      for t in pytd_utils.GetTypeParameters(return_type):
         if t.full_name not in subst:
           subst[t.full_name] = self.vm.convert.empty.to_variable(node)
       return node, self.vm.convert.constant_to_var(
@@ -773,6 +769,15 @@ class PyTDSignature(utils.VirtualMachineWeakrefMixin):
     """
     # Handle mutable parameters using the information type parameters
     mutations = []
+    # It's possible that the signature contains type parameters that are used
+    # in mutations but are not filled in by the arguments, e.g. when starargs
+    # and starstarargs have type parameters but are not in the args. Check that
+    # subst has an entry for every type parameter, adding any that are missing.
+    if any(f.mutated_type for f in self.pytd_sig.params):
+      subst = subst.copy()
+      for t in pytd_utils.GetTypeParameters(self.pytd_sig):
+        if t.full_name not in subst:
+          subst[t.full_name] = self.vm.convert.empty.to_variable(node)
     for formal in self.pytd_sig.params:
       actual = arg_dict[formal.name]
       arg = actual.data
