@@ -24,7 +24,7 @@ PUSHES_BLOCK = 2048  # starts a block (while, try, finally, with, etc.)
 POPS_BLOCK = 4096  # ends a block
 
 
-class Opcode(object):
+class Opcode:
   """An opcode without arguments."""
 
   __slots__ = ("line", "index", "prev", "next", "target", "block_target",
@@ -138,7 +138,7 @@ class OpcodeWithArg(Opcode):
   __slots__ = ("arg", "pretty_arg")
 
   def __init__(self, index, line, arg, pretty_arg):
-    super(OpcodeWithArg, self).__init__(index, line)
+    super().__init__(index, line)
     self.arg = arg
     self.pretty_arg = pretty_arg
 
@@ -319,6 +319,22 @@ class BEFORE_ASYNC_WITH(Opcode):
   __slots__ = ()
 
 
+class BEGIN_FINALLY(Opcode):
+  __slots__ = ()
+
+
+class END_ASYNC_FOR(Opcode):
+  # Even though dis documentation says that END_ASYNC_FOR may reraise an
+  # exception, we do not include NO_NEXT in the flags because doing so would
+  # cause the return statement for an async method to be skipped, leading to
+  # an incorrect return type.
+  # See tests/py3/test_stdlib:StdlibTestsFeatures.test_async_iter and
+  # tests/py3/test_coroutine:GeneratorFeatureTest.test_async_for_pyi for tests
+  # that fail if we add NO_NEXT.
+  FLAGS = HAS_JUNKNOWN
+  __slots__ = ()
+
+
 class STORE_MAP(Opcode):
   __slots__ = ()
 
@@ -831,6 +847,68 @@ class LOAD_METHOD(OpcodeWithArg):  # Arg: Index in name list
 
 
 class CALL_METHOD(OpcodeWithArg):  # Arg: #args
+  FLAGS = HAS_NARGS|HAS_ARGUMENT|HAS_JUNKNOWN
+  __slots__ = ()
+
+
+class CALL_FINALLY(OpcodeWithArg):  # Arg: Jump offset to finally block
+  FLAGS = HAS_JREL | HAS_ARGUMENT
+  __slots__ = ()
+
+
+class POP_FINALLY(OpcodeWithArg):
+  # might re-raise an exception or jump to a finally
+  FLAGS = HAS_ARGUMENT | HAS_JUNKNOWN
+  __slots__ = ()
+
+
+class RERAISE(Opcode):
+  __slots__ = ()
+
+
+class WITH_EXCEPT_START(Opcode):
+  __slots__ = ()
+
+
+class LOAD_ASSERTION_ERROR(Opcode):
+  __slots__ = ()
+
+
+class LIST_TO_TUPLE(Opcode):
+  __slots__ = ()
+
+
+class IS_OP(OpcodeWithArg):
+  FLAGS = HAS_ARGUMENT
+  __slots__ = ()
+
+
+class CONTAINS_OP(OpcodeWithArg):
+  FLAGS = HAS_ARGUMENT
+  __slots__ = ()
+
+
+class JUMP_IF_NOT_EXC_MATCH(OpcodeWithArg):
+  FLAGS = HAS_ARGUMENT | HAS_JABS
+  __slots__ = ()
+
+
+class LIST_EXTEND(OpcodeWithArg):
+  FLAGS = HAS_ARGUMENT
+  __slots__ = ()
+
+
+class SET_UPDATE(OpcodeWithArg):
+  FLAGS = HAS_ARGUMENT
+  __slots__ = ()
+
+
+class DICT_MERGE(OpcodeWithArg):
+  FLAGS = HAS_ARGUMENT
+  __slots__ = ()
+
+
+class DICT_UPDATE(OpcodeWithArg):
   FLAGS = HAS_ARGUMENT
   __slots__ = ()
 
@@ -1078,7 +1156,7 @@ python_3_5_mapping = {
 def _overlay_mapping(mapping, new_entries):
   ret = mapping.copy()
   ret.update(new_entries)
-  return dict((k, v) for k, v in six.iteritems(ret) if v is not None)
+  return dict((k, v) for k, v in ret.items() if v is not None)
 
 python_3_6_mapping = _overlay_mapping(python_3_5_mapping, {
     85: SETUP_ANNOTATIONS,
@@ -1097,16 +1175,52 @@ python_3_7_mapping = _overlay_mapping(python_3_6_mapping, {
     161: CALL_METHOD,
 })
 
+python_3_8_mapping = _overlay_mapping(python_3_7_mapping, {
+    6: ROT_FOUR,  # ROT_FOUR returns under a different, cooler id!
+    53: BEGIN_FINALLY,
+    54: END_ASYNC_FOR,
+    80: None,   # BREAK_LOOP was removed in 3.8
+    119: None,  # CONTINUE_LOOP was removed in 3.8
+    120: None,  # SETUP_LOOP was removed in 3.8
+    121: None,  # SETUP_EXCEPT was removed in 3.8
+    162: CALL_FINALLY,
+    163: POP_FINALLY,
+})
 
-class _LineNumberTableParser(object):
+python_3_9_mapping = _overlay_mapping(python_3_8_mapping, {
+    48: RERAISE,
+    49: WITH_EXCEPT_START,
+    53: None,  # was BEGIN_FINALLY in 3.8
+    74: LOAD_ASSERTION_ERROR,
+    81: None,  # was WITH_CLEANUP_START in 3.8
+    82: LIST_TO_TUPLE,  # was WITH_CLEANUP_FINISH in 3.8
+    88: None,  # was END_FINALLY in 3.8
+    117: IS_OP,
+    118: CONTAINS_OP,
+    121: JUMP_IF_NOT_EXC_MATCH,
+    149: None,  # was BUILD_LIST_UNPACK in 3.8
+    150: None,  # was BUILD_MAP_UNPACK in 3.8
+    151: None,  # was BUILD_MAP_UNPACK_WITH_CALL in 3.8
+    152: None,  # was BUILD_TUPLE_UNPACK in 3.8
+    153: None,  # was BUILD_SET_UNPACK in 3.8
+    158: None,  # was BUILD_TUPLE_UNPACK_WITH_CALL in 3.8
+    162: LIST_EXTEND,  # was CALL_FINALLY in 3.8
+    163: SET_UPDATE,  # was POP_FINALLY in 3.8
+    164: DICT_MERGE,
+    165: DICT_UPDATE,
+})
+
+
+class _LineNumberTableParser:
   """State machine for decoding a Python line number array."""
 
-  def __init__(self, lnotab, firstlineno):
+  def __init__(self, python_version, lnotab, firstlineno):
     assert not len(lnotab) & 1  # lnotab always has an even number of elements
     self.lnotab = lnotab
     self.lineno = firstlineno
     self.next_addr = six.indexbytes(self.lnotab, 0) if self.lnotab else 0
     self.pos = 0
+    self.python_version = python_version
 
   def get(self, i):
     """Get the line number for the instruction at the given position.
@@ -1121,7 +1235,14 @@ class _LineNumberTableParser(object):
       The line number corresponding to the position at i.
     """
     while i >= self.next_addr and self.pos < len(self.lnotab):
-      self.lineno += six.indexbytes(self.lnotab, self.pos + 1)
+      line_diff = six.indexbytes(self.lnotab, self.pos + 1)
+      # The Python docs have more details on this weird bit twiddling.
+      # https://github.com/python/cpython/blob/master/Objects/lnotab_notes.txt
+      # https://github.com/python/cpython/commit/f3914eb16d28ad9eb20fe5133d9aa83658bcc27f
+      if self.python_version >= (3, 6) and line_diff >= 0x80:
+        line_diff -= 0x100
+      self.lineno += line_diff
+
       self.pos += 2
       if self.pos < len(self.lnotab):
         self.next_addr += six.indexbytes(self.lnotab, self.pos)
@@ -1222,12 +1343,15 @@ def _wordcode_reader(data, mapping):
       start = pos + 2
 
 
-def _dis(data, mapping, reader,
+def _dis(python_version, data, mapping, reader,
          co_varnames=None, co_names=None, co_consts=None, co_cellvars=None,
          co_freevars=None, co_lnotab=None, co_firstlineno=None):
   """Disassemble a string into a list of Opcode instances."""
   code = []
-  lp = _LineNumberTableParser(co_lnotab, co_firstlineno) if co_lnotab else None
+  if co_lnotab:
+    lp = _LineNumberTableParser(python_version, co_lnotab, co_firstlineno)
+  else:
+    lp = None
   offset_to_index = {}
   if co_cellvars is not None and co_freevars is not None:
     cellvars_freevars = co_cellvars + co_freevars
@@ -1271,9 +1395,11 @@ def dis(data, python_version, *args, **kwargs):
       (3, 5): python_3_5_mapping,
       (3, 6): python_3_6_mapping,
       (3, 7): python_3_7_mapping,
+      (3, 8): python_3_8_mapping,
+      (3, 9): python_3_9_mapping,
   }[(major, minor)]
   reader = _wordcode_reader if (major, minor) > (3, 5) else _bytecode_reader
-  return _dis(data, mapping, reader, *args, **kwargs)
+  return _dis(python_version, data, mapping, reader, *args, **kwargs)
 
 
 def dis_code(code):
