@@ -13,10 +13,11 @@ from pytype.pytd import builtins
 import toml
 
 
-def _get_module_names_in_path(lister, path):
+def _get_module_names_in_path(lister, path, python_version):
+  """Get module names for all .pyi files in the given path."""
   names = set()
   try:
-    contents = list(lister(path))
+    contents = list(lister(path, python_version))
   except pytype_source_utils.NoSuchDirectory:
     pass
   else:
@@ -50,15 +51,8 @@ class Typeshed:
     else:
       self._root = pytype_source_utils.get_full_path("typeshed")
     self._missing = frozenset(self._load_missing())
-    # See https://github.com/google/pytype/issues/820. typeshed's directory
-    # structure significantly changed in January 2021. We need to support both
-    # the old and the new structures until our bundled typeshed is updated past
-    # the restructuring commit.
-    self._use_new_structure = os.path.exists(
-        os.path.join(self._root, "stdlib", "VERSIONS"))
-    if self._use_new_structure:
-      self._stdlib_versions = self._load_stdlib_versions()
-      self._third_party_packages = self._load_third_party_packages()
+    self._stdlib_versions = self._load_stdlib_versions()
+    self._third_party_packages = self._load_third_party_packages()
 
   def _load_file(self, path):
     if self._env_home:
@@ -69,6 +63,37 @@ class Typeshed:
       filepath = os.path.join(self._root, path)
       return filepath, pytype_source_utils.load_text_file(filepath)
 
+  def _file_exists(self, relpath):
+    """Checks whether the given path, relative to the typeshed root, exists."""
+    if self._env_home:
+      return os.path.exists(os.path.join(self._root, relpath))
+    try:
+      # For a non-par pytype installation, load_text_file will either succeed,
+      # raise FileNotFoundError, or raise IsADirectoryError.
+      # For a par installation, load_text_file will raise FileNotFoundError for
+      # both a nonexistent file and a directory.
+      pytype_source_utils.load_text_file(os.path.join("typeshed", relpath))
+    except FileNotFoundError:
+      try:
+        # For a non-par installation, we know at this point that relpath does
+        # not exist, so _list_files will always raise NoSuchDirectory. For a par
+        # installation, we use _list_files to check whether the directory
+        # exists; a non-existent directory will produce an empty generator.
+        next(self._list_files(relpath))
+      except (pytype_source_utils.NoSuchDirectory, StopIteration):
+        return False
+    except IsADirectoryError:
+      return True
+    return True
+
+  def _list_files(self, basedir):
+    """Lists files recursively in a basedir relative to typeshed root."""
+    if self._env_home:
+      return pytype_source_utils.list_files(os.path.join(self._root, basedir))
+    else:
+      return pytype_source_utils.list_pytype_files(
+          os.path.join("typeshed", basedir))
+
   def _load_missing(self):
     if not self.MISSING_FILE:
       return set()
@@ -109,20 +134,24 @@ class Typeshed:
       A mapping from module name to a set of
       (package name, major_python_version) tuples.
     """
-    third_party_root = os.path.join(self._root, "stubs")
+    metadata = {}
+    modules = collections.defaultdict(set)
+    for third_party_file in self._list_files("stubs"):
+      parts = third_party_file.split(os.path.sep)
+      if parts[-1] == "METADATA.toml":  # {package}/METADATA.toml
+        _, metadata_file = self._load_file(
+            os.path.join("stubs", third_party_file))
+        metadata[parts[0]] = toml.loads(metadata_file)
+      elif "@python2" not in parts:  # {package}/{module}
+        name, _ = os.path.splitext(parts[1])
+        modules[parts[0]].add(name)
     packages = collections.defaultdict(set)
-    for package in os.listdir(third_party_root):
-      _, metadata = self._load_file(
-          os.path.join(third_party_root, package, "METADATA.toml"))
-      metadata = toml.loads(metadata)
-      for name in os.listdir(os.path.join(third_party_root, package)):
-        if name in ("METADATA.toml", "@python2"):
-          continue
-        name, _ = os.path.splitext(name)
-        # When not specified, packages are Python 3-only.
-        if metadata.get("python2", False):
+    for package, names in modules.items():
+      for name in names:
+        # When not specified, packages are Python 3-only
+        if metadata[package].get("python2", False):
           packages[name].add((package, 2))
-        if metadata.get("python3", True):
+        if metadata[package].get("python3", True):
           packages[name].add((package, 3))
     return packages
 
@@ -166,13 +195,6 @@ class Typeshed:
     Raises:
       IOError: if file not found
     """
-    if self._use_new_structure:
-      return self._get_module_file(toplevel, module, version)
-    else:
-      return self._get_module_file_old(toplevel, module, version)
-
-  def _get_module_file(self, toplevel, module, version):
-    """get_module_file for typeshed's new directory structure."""
     module_parts = module.split(".")
     module_path = os.path.join(*module_parts)
     paths = []
@@ -181,10 +203,12 @@ class Typeshed:
       # (for Python 2) stdlib/@python2/foo. The VERSIONS file tells us whether
       # stdlib/foo exists and what versions it targets; we also have to
       # separately check for stdlib/@python2/foo.
-      if (module_parts[0] in self._stdlib_versions and
-          self._stdlib_versions[module_parts[0]] <= version):
-        paths.append(os.path.join(toplevel, module_path))
-      elif version[0] == 2:
+      path = os.path.join(toplevel, module_path)
+      if ((module_parts[0] in self._stdlib_versions and
+           self._stdlib_versions[module_parts[0]] <= version) or
+          path in self.missing):
+        paths.append(path)
+      if version[0] == 2:
         paths.append(os.path.join(toplevel, "@python2", module_path))
     elif toplevel == "third_party":
       # For third-party modules, we grab the alphabetically first package that
@@ -215,44 +239,8 @@ class Typeshed:
           pass
     raise IOError("Couldn't find %s" % module)
 
-  def _get_module_file_old(self, toplevel, module, version):
-    """get_module_file for typeshed's old directory structure."""
-    if self._ignore(module, version):
-      raise IOError("Couldn't find %s" % module)
-    module_path = os.path.join(*module.split("."))
-    versions = ["%d.%d" % (version[0], minor)
-                for minor in range(version[1], -1, -1)]
-    # E.g. for Python 3.5, try 3.5/, 3.4/, 3.3/, ..., 3.0/, 3/, 2and3.
-    # E.g. for Python 2.7, try 2.7/, 2.6/, ..., 2/, 2and3.
-    # The order is the same as that of mypy. See default_lib_path in
-    # https://github.com/JukkaL/mypy/blob/master/mypy/build.py#L249
-    for v in versions + [str(version[0]), "2and3"]:
-      path_rel = os.path.join(toplevel, v, module_path)
-
-      # Give precedence to MISSING_FILE
-      if path_rel in self.missing:
-        return (os.path.join(self._root, "nonexistent", path_rel + ".pyi"),
-                builtins.DEFAULT_SRC)
-
-      # TODO(mdemello): handle this in the calling code.
-      for path in [os.path.join(path_rel, "__init__.pyi"), path_rel + ".pyi"]:
-        try:
-          name, src = self._load_file(path)
-          return name, src
-        except IOError:
-          pass
-
-    raise IOError("Couldn't find %s" % module)
-
   def get_typeshed_paths(self, python_version):
     """Gets the paths to typeshed's version-specific pyi files."""
-    if self._use_new_structure:
-      return self._get_typeshed_paths(python_version)
-    else:
-      return self._get_typeshed_paths_old(python_version)
-
-  def _get_typeshed_paths(self, python_version):
-    """get_typeshed_paths for typeshed's new directory structure."""
     major, _ = python_version
     typeshed_subdirs = ["stdlib"]
     if major == 2:
@@ -260,23 +248,11 @@ class Typeshed:
     for packages in self._third_party_packages.values():
       for package, v in packages:
         if v == major:
-          typeshed_subdirs.append(os.path.join("stubs", package))
-          if v == 2:
-            typeshed_subdirs.append(os.path.join("stubs", package, "@python2"))
-    return [os.path.join(self._root, d) for d in typeshed_subdirs]
-
-  def _get_typeshed_paths_old(self, python_version):
-    """get_typeshed_paths for typeshed's old directory structure."""
-    major, minor = python_version
-    typeshed_subdirs = ["stdlib/%d" % major,
-                        "stdlib/2and3",
-                        "third_party/%d" % major,
-                        "third_party/2and3",
-                       ]
-    if major == 3:
-      for i in range(0, minor + 1):
-        # iterate over 3.0, 3.1, 3.2, ...
-        typeshed_subdirs.append("stdlib/3.%d" % i)
+          py2only_dir = os.path.join("stubs", package, "@python2")
+          if v == 2 and self._file_exists(py2only_dir):
+            typeshed_subdirs.append(os.path.join(py2only_dir))
+          else:
+            typeshed_subdirs.append(os.path.join("stubs", package))
     return [os.path.join(self._root, d) for d in typeshed_subdirs]
 
   def get_pytd_paths(self, python_version):
@@ -286,28 +262,57 @@ class Typeshed:
         "stubs/builtins/%d" % python_version[0],
         "stubs/stdlib/%d" % python_version[0]]]
 
+  def _list_modules(self, path, python_version):
+    """Lists modules for _get_module_names_in_path."""
+    for filename in self._list_files(path):
+      if filename.startswith("@python2/"):
+        # When Python 2 is requested, relative paths that already have the
+        # @python2/ prefix stripped are separately listed, so we should always
+        # skip paths that start with @python2/.
+        continue
+      if filename == "VERSIONS" or filename == "METADATA.toml":
+        # stdlib/VERSIONS, stubs/{package}/METADATA.toml are metadata files.
+        continue
+      parts = path.split("/")
+      if "stdlib" in parts:
+        if "@python2" in parts:
+          # stdlib/@python2/ stubs are Python 2-only.
+          if python_version[0] != 2:
+            continue
+        else:
+          # Check supported versions for stubs directly in stdlib/.
+          module = os.path.splitext(filename)[0].split("/", 1)[0]
+          if (module not in self._stdlib_versions or
+              self._stdlib_versions[module] > python_version):
+            continue
+      yield filename
+
   def get_all_module_names(self, python_version):
     """Get the names of all modules in typeshed or bundled with pytype."""
     module_names = set()
-    typeshed_paths = self.get_typeshed_paths(python_version)
-    pytd_paths = self.get_pytd_paths(python_version)
-    if self._env_home:
-      for p in typeshed_paths:
-        module_names |= _get_module_names_in_path(
-            pytype_source_utils.list_files, p)
-      pytype_paths = pytd_paths
-    else:
-      pytype_paths = typeshed_paths + pytd_paths
-    subdirs = [d.rpartition("pytype/")[-1] for d in pytype_paths]
-    for subdir in subdirs:
+    for abspath in self.get_typeshed_paths(python_version):
+      relpath = abspath.rpartition("typeshed/")[-1]
       module_names |= _get_module_names_in_path(
-          pytype_source_utils.list_pytype_files, subdir)
+          self._list_modules, relpath, python_version)
+    for abspath in self.get_pytd_paths(python_version):
+      relpath = abspath.rpartition("pytype/")[-1]
+      module_names |= _get_module_names_in_path(
+          lambda path, unused_ver: pytype_source_utils.list_pytype_files(path),
+          relpath, python_version)
     # Also load modules not in typeshed, so that we have a dummy entry for them.
     for f in self.missing:
       parts = f.split("/")
-      if parts[1].startswith(str(python_version[0])):
-        filename = "/".join(parts[2:])  # remove prefixes like stdlib/2.7
-        module_names.add(filename.replace("/", "."))
+      if ("@python2" in parts) != (python_version[0] == 2):
+        continue
+      if parts[0] == "stdlib":
+        start_index = 1  # remove stdlib/ prefix
+      else:
+        assert parts[0] == "stubs"
+        start_index = 2  # remove stubs/{package}/ prefix
+      if parts[start_index] == "@python2":
+        start_index += 1
+      filename = "/".join(parts[start_index:])
+      module_names.add(filename.replace("/", "."))
     assert "ctypes" in module_names  # sanity check
     return module_names
 
@@ -331,13 +336,6 @@ class Typeshed:
 
   def get_python_major_versions(self, filename):
     """Gets the Python major versions targeted by the given .pyi file."""
-    if self._use_new_structure:
-      return self._get_python_major_versions(filename)
-    else:
-      return self._get_python_major_versions_old(filename)
-
-  def _get_python_major_versions(self, filename):
-    """get_python_major_versions for the new typeshed directory structure."""
     if os.path.sep + "@python2" + os.path.sep in filename:
       return (2,)
     parts = filename.split(os.path.sep)
@@ -351,22 +349,14 @@ class Typeshed:
       package, module = parts[1], os.path.splitext(parts[2])[0]
       versions = []
       for p, v in self._third_party_packages[module]:
-        if p != package or v == 2 and os.path.exists(
-            os.path.join(self._root, "stubs", p, "@python2")):
+        if p != package or v == 2 and self._file_exists(
+            os.path.join("stubs", p, "@python2")):
           # If a dedicated @python2 subdirectory exists, then the top-level
           # stubs are Python 3-only.
           continue
         versions.append(v)
       return tuple(versions)
 
-  def _get_python_major_versions_old(self, filename):
-    """get_python_major_versions for the old typeshed directory structure."""
-    path = filename.split(os.path.sep)
-    if path[1] == "2and3":
-      return (2, 3)
-    else:
-      return (int(path[1][0]),)
-
 
 _typeshed = None
 
