@@ -15,6 +15,7 @@ from pytype.pytd import base_visitor
 from pytype.pytd import escape
 from pytype.pytd import mro
 from pytype.pytd import pep484
+from pytype.pytd import printer
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import pytd_visitors
@@ -37,23 +38,10 @@ CanonicalOrderingVisitor = pytd_visitors.CanonicalOrderingVisitor
 ClassTypeToNamedType = pytd_visitors.ClassTypeToNamedType
 CollectTypeParameters = pytd_visitors.CollectTypeParameters
 ExtractSuperClasses = pytd_visitors.ExtractSuperClasses
-PrintVisitor = pytd_visitors.PrintVisitor
+PrintVisitor = printer.PrintVisitor
 RenameModuleVisitor = pytd_visitors.RenameModuleVisitor
 
 
-def InventStarArgParams(existing_names):
-  """Try to find names for *args, **kwargs that aren't taken already."""
-  names = {x if isinstance(x, str) else x.name
-           for x in existing_names}
-  args, kwargs = "args", "kwargs"
-  while args in names:
-    args = "_" + args
-  while kwargs in names:
-    kwargs = "_" + kwargs
-  return (pytd.Parameter(args, pytd.NamedType("tuple"), False, True, None),
-          pytd.Parameter(kwargs, pytd.NamedType("dict"), False, True, None))
-
-
 class StripSelf(Visitor):
   """Transforms the tree into one where methods don't have the "self" parameter.
 
@@ -348,25 +336,20 @@ class LookupBuiltins(_ToTypeVisitor):
 
   def VisitNamedType(self, t):
     """Do lookup on a pytd.NamedType."""
-    if "." in t.name:
+    if "." in t.name or self._prefix + t.name in self._current_unit:
       return t
+    # We can't find this identifier in our current module, and it isn't fully
+    # qualified (doesn't contain a dot). Now check whether it's a builtin.
     try:
-      self._current_unit.Lookup(self._prefix + t.name)
+      item = self._builtins.Lookup(self._builtins.name + "." + t.name)
     except KeyError:
-      # We can't find this identifier in our current module, and it isn't fully
-      # qualified (doesn't contain a dot). Now check whether it's a builtin.
-      try:
-        item = self._builtins.Lookup(self._builtins.name + "." + t.name)
-      except KeyError:
-        return t
-      else:
-        try:
-          return self.to_type(item)
-        except NotImplementedError:
-          # This can happen if a builtin is redefined.
-          return t
-    else:
       return t
+    else:
+      try:
+        return self.to_type(item)
+      except NotImplementedError:
+        # This can happen if a builtin is redefined.
+        return t
 
 
 def MaybeSubstituteParameters(base_type, parameters=None):
@@ -387,6 +370,40 @@ def MaybeSubstituteParameters(base_type, parameters=None):
   return base_type.Visit(ReplaceTypeParameters(mapping))
 
 
+def _LookupItemRecursive(module, module_name, name):
+  """Recursively look up name in module."""
+  parts = name.split(".")
+  partial_name = module_name
+  prev_item = None
+  item = module
+  for part in parts:
+    prev_item = item
+    # Check the type of item and give up if we encounter a type we don't know
+    # how to handle.
+    if isinstance(item, pytd.Constant):
+      if not isinstance(item.type, pytd.ClassType):
+        raise KeyError(name)
+      item = item.type.cls
+    elif not isinstance(item, (pytd.TypeDeclUnit, pytd.Class)):
+      raise KeyError(name)
+    lookup_name = partial_name + "." + part
+    # Nested class names are fully qualified while function names are not, so
+    # we try lookup for both naming conventions.
+    try:
+      item = item.Lookup(lookup_name)
+    except KeyError:
+      item = item.Lookup(part)
+    if isinstance(item, pytd.Constant):
+      partial_name += "." + item.name.rsplit(".", 1)[-1]
+    else:
+      partial_name = lookup_name
+  if isinstance(item, pytd.Function):
+    return pytd_utils.AliasMethod(
+        item, from_constant=isinstance(prev_item, pytd.Constant))
+  else:
+    return item
+
+
 class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
   """Look up NamedType pointers using a symbol table."""
 
@@ -408,6 +425,7 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     self._alias_name = None
     self._in_generic_type = 0
     self._star_imports = set()
+    self._unit = None
 
   def _ResolveUsingGetattr(self, module_name, module):
     """Try to resolve an identifier using the top level __getattr__ function."""
@@ -445,15 +463,6 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
     else:
       raise KeyError("Unknown module %s" % name)
 
-  def _LookupItemRecursive(self, module, module_name, name):
-    parts = name.split(".")
-    partial_name = module_name
-    item = module
-    for part in parts:
-      partial_name += "." + part
-      item = item.Lookup(partial_name)
-    return item
-
   def VisitNamedType(self, t):
     """Try to look up a NamedType.
 
@@ -479,14 +488,20 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
       return t
     if module_name in self._module_alias_map:
       module_name = self._module_alias_map[module_name]
-    module, module_name, cls_prefix = self._LookupModuleRecursive(module_name)
+    try:
+      module, module_name, cls_prefix = self._LookupModuleRecursive(module_name)
+    except KeyError:
+      if self._unit and f"{self.name}.{module_name}" in self._unit:
+        # Nothing to do here.This is a dotted local reference.
+        return t
+      raise
     name = cls_prefix + name
     try:
       if name == "*":
         self._star_imports.add(module_name)
         item = t  # VisitTypeDeclUnit will remove this unneeded item.
       else:
-        item = self._LookupItemRecursive(module, module_name, name)
+        item = _LookupItemRecursive(module, module_name, name)
     except KeyError as e:
       item = self._ResolveUsingGetattr(module_name, module)
       if item is None:
@@ -546,9 +561,7 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
   def _DiscardExistingNames(self, node, potential_members):
     new_members = []
     for m in potential_members:
-      try:
-        node.Lookup(m.name)
-      except KeyError:
+      if m.name not in node:
         new_members.append(m)
     return new_members
 
@@ -581,6 +594,9 @@ class LookupExternalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
         out.append(a)
     return out
 
+  def EnterTypeDeclUnit(self, node):
+    self._unit = node
+
   def VisitTypeDeclUnit(self, node):
     """Add star imports to the ast.
 
@@ -632,10 +648,13 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
   def LeaveTypeDeclUnit(self, _):
     del self.unit
 
+  def _LookupItemRecursive(self, name):
+    return _LookupItemRecursive(self.unit, self.unit.name, name)
+
   def VisitNamedType(self, node):
     """Do lookup on a pytd.NamedType."""
-    module_name, dot, _ = node.name.rpartition(".")
-    if not dot:
+    module_name, dot, name = node.name.rpartition(".")
+    if not dot:  # simple reference to a member of the current module
       try:
         item = self.unit.Lookup(self.unit.name + "." + node.name)
       except KeyError:
@@ -647,9 +666,16 @@ class LookupLocalTypes(RemoveTypeParametersFromGenericAny, _ToTypeVisitor):
               node.name, self.unit.name)) from e
       resolved_node = self.to_type(item)
     elif module_name == self.unit.name:
-      resolved_node = self.to_type(self.unit.Lookup(node.name))
+      # Fully qualified reference to a member of the current module. May contain
+      # nested items that need to be recursively resolved.
+      resolved_node = self.to_type(self._LookupItemRecursive(name))
     else:
-      resolved_node = node
+      # Possibly a reference to a member of the current module that does not
+      # have a module_name prefix.
+      try:
+        resolved_node = self.to_type(self._LookupItemRecursive(node.name))
+      except KeyError:
+        resolved_node = node  # lookup failures are handled later
     if isinstance(resolved_node, (pytd.Constant, pytd.Function)):
       visitor = LookupLocalTypes()
       visitor.unit = self.unit
