@@ -1,5 +1,4 @@
 """Integration test for pytype."""
-from __future__ import print_function
 
 import csv
 import hashlib
@@ -14,10 +13,11 @@ from pytype import config
 from pytype import single
 from pytype import utils
 from pytype.pyi import parser
+from pytype.pytd import builtins
 from pytype.pytd import pytd_utils
 from pytype.pytd import typeshed
-from pytype.pytd.parse import builtins
 from pytype.tests import test_base
+
 import unittest
 
 
@@ -29,17 +29,17 @@ class PytypeTest(test_base.UnitTest):
 
   @classmethod
   def setUpClass(cls):
-    super(PytypeTest, cls).setUpClass()
+    super().setUpClass()
     cls.pytype_dir = os.path.dirname(os.path.dirname(parser.__file__))
 
   def setUp(self):
-    super(PytypeTest, self).setUp()
+    super().setUp()
     self._reset_pytype_args()
     self.tmp_dir = tempfile.mkdtemp()
     self.errors_csv = os.path.join(self.tmp_dir, "errors.csv")
 
   def tearDown(self):
-    super(PytypeTest, self).tearDown()
+    super().tearDown()
     shutil.rmtree(self.tmp_dir)
 
   def _reset_pytype_args(self):
@@ -197,6 +197,17 @@ class PytypeTest(test_base.UnitTest):
     self.assertTrue(os.path.exists(pickled_location))
     return pickled_location
 
+  def test_run_pytype(self):
+    """Basic unit test (smoke test) for _run_pytype."""
+    # Note: all other tests in this file are integration tests.
+    infile = self._tmp_path("input")
+    outfile = self._tmp_path("output")
+    with open(infile, "w") as f:
+      f.write("def f(x): pass")
+    options = config.Options.create(infile, output=outfile)
+    single._run_pytype(options)
+    self.assertTrue(os.path.isfile(outfile))
+
   def test_pickled_file_stableness(self):
     # Tests that the pickled format is stable under a constant PYTHONHASHSEED.
     l_1 = self.generate_pickled_simple_file("simple1.pickled")
@@ -456,18 +467,6 @@ class PytypeTest(test_base.UnitTest):
     self._infer_types_and_check_errors(self._data_path(filename),
                                        ["bad-return-type"])
 
-  def test_run_pytype(self):
-    """Basic unit test (smoke test) for _run_pytype."""
-    # TODO(kramm): This is a unit test, whereas all other tests in this file
-    # are integration tests. Move this somewhere else?
-    infile = self._tmp_path("input")
-    outfile = self._tmp_path("output")
-    with open(infile, "w") as f:
-      f.write("def f(x): pass")
-    options = config.Options.create(infile, output=outfile)
-    single._run_pytype(options)
-    self.assertTrue(os.path.isfile(outfile))
-
   def test_generate_and_use_builtins(self):
     """Test for --generate-builtins."""
     filename = self._tmp_path("builtins.pickle")
