@@ -1,6 +1,5 @@
 """Tests for vm.py."""
 
-import dis
 import textwrap
 
 from pytype import analyze
@@ -9,18 +8,21 @@ from pytype import compat
 from pytype import errors
 from pytype import utils
 from pytype import vm
+from pytype.pyc import opcodes
 from pytype.pyc import pyc
 from pytype.tests import test_base
 from pytype.tests import test_utils
 
 import six
 
+_PY2_OPMAP = {v.__name__: k for k, v in opcodes.python2_mapping.items()}
+
 
 class TraceVM(vm.VirtualMachine):
   """Special VM that remembers which instructions it executed."""
 
   def __init__(self, options, loader):
-    super(TraceVM, self).__init__(errors.ErrorLog(), options, loader=loader)
+    super().__init__(errors.ErrorLog(), options, loader=loader)
     # There are multiple possible orderings of the basic blocks of the code, so
     # we collect the instructions in an order-independent way:
     self.instructions_executed = set()
@@ -32,7 +34,7 @@ class TraceVM(vm.VirtualMachine):
 
   def run_instruction(self, op, state):
     self.instructions_executed.add(op.index)
-    return super(TraceVM, self).run_instruction(op, state)
+    return super().run_instruction(op, state)
 
 
 class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
@@ -42,7 +44,7 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
   PY_MAJOR_VERSIONS = [2]
 
   def setUp(self):
-    super(BytecodeTest, self).setUp()
+    super().setUp()
     self.errorlog = errors.ErrorLog()
     self.trace_vm = TraceVM(self.options, self.loader)
 
@@ -53,7 +55,7 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
         0x64, 1, 0,  # 0 LOAD_CONST, arg=1 (1)
         0x53,  # 3 RETURN_VALUE
     ], name="simple")
-    code = blocks.process_code(code)
+    code = blocks.process_code(code, self.python_version)
     v = vm.VirtualMachine(self.errorlog, self.options, loader=self.loader)
     v.run_bytecode(v.program.NewCFGNode(), code)
 
@@ -86,7 +88,7 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
         o.LOAD_FAST, 1, 0,
         o.RETURN_VALUE,
     ])
-    code = blocks.process_code(code)
+    code = blocks.process_code(code, self.python_version)
     v = vm.VirtualMachine(self.errorlog, self.options, loader=self.loader)
     v.run_bytecode(v.program.NewCFGNode(), code)
 
@@ -99,38 +101,38 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
           z += x*a
     """)
   code_nested_loop = compat.int_array_to_bytes([
-      dis.opmap["LOAD_CONST"], 0, 0,          # [0], 0, arg=0
-      dis.opmap["LOAD_CONST"], 1, 0,          # [1], 3, arg=1
-      dis.opmap["LOAD_CONST"], 2, 0,          # [2], 6, arg=2
-      dis.opmap["BUILD_LIST"], 3, 0,          # [3], 9, arg=3
-      dis.opmap["STORE_NAME"], 0, 0,          # [4], 12, arg=0
-      dis.opmap["LOAD_CONST"], 3, 0,          # [5], 15, arg=3
-      dis.opmap["STORE_NAME"], 1, 0,          # [6], 18, arg=1
-      dis.opmap["SETUP_LOOP"], 54, 0,         # [7], 21, dest=78
-      dis.opmap["LOAD_NAME"], 0, 0,           # [8], 24, arg=0
-      dis.opmap["GET_ITER"],                  # [9], 27
-      dis.opmap["FOR_ITER"], 46, 0,           # [10], 28, dest=77
-      dis.opmap["STORE_NAME"], 2, 0,          # [11], 31, arg=2
-      dis.opmap["SETUP_LOOP"], 37, 0,         # [12], 34, dest=74
-      dis.opmap["LOAD_NAME"], 0, 0,           # [13], 37, arg=0
-      dis.opmap["GET_ITER"],                  # [14], 40
-      dis.opmap["FOR_ITER"], 29, 0,           # [15], 41, dest=73
-      dis.opmap["STORE_NAME"], 3, 0,          # [16], 44, arg=3
-      dis.opmap["LOAD_NAME"], 2, 0,           # [17], 47, arg=2
-      dis.opmap["POP_JUMP_IF_FALSE"], 41, 0,  # [18], 50, dest=41
-      dis.opmap["LOAD_NAME"], 1, 0,           # [19], 53, arg=1
-      dis.opmap["LOAD_NAME"], 2, 0,           # [20], 56, arg=2
-      dis.opmap["LOAD_NAME"], 3, 0,           # [21], 59, arg=3
-      dis.opmap["BINARY_MULTIPLY"],           # [22], 62
-      dis.opmap["INPLACE_ADD"],               # [23], 63
-      dis.opmap["STORE_NAME"], 1, 0,          # [24], 64, arg=1
-      dis.opmap["JUMP_ABSOLUTE"], 41, 0,      # [25], 67, dest=41
-      dis.opmap["JUMP_ABSOLUTE"], 41, 0,      # [26], 70 (unreachable), dest=41
-      dis.opmap["POP_BLOCK"],                 # [27], 73
-      dis.opmap["JUMP_ABSOLUTE"], 28, 0,      # [28], 74, dest=28
-      dis.opmap["POP_BLOCK"],                 # [29], 77
-      dis.opmap["LOAD_CONST"], 4, 0,          # [30], 78, arg=4
-      dis.opmap["RETURN_VALUE"],              # [31], 81
+      _PY2_OPMAP["LOAD_CONST"], 0, 0,          # [0], 0, arg=0
+      _PY2_OPMAP["LOAD_CONST"], 1, 0,          # [1], 3, arg=1
+      _PY2_OPMAP["LOAD_CONST"], 2, 0,          # [2], 6, arg=2
+      _PY2_OPMAP["BUILD_LIST"], 3, 0,          # [3], 9, arg=3
+      _PY2_OPMAP["STORE_NAME"], 0, 0,          # [4], 12, arg=0
+      _PY2_OPMAP["LOAD_CONST"], 3, 0,          # [5], 15, arg=3
+      _PY2_OPMAP["STORE_NAME"], 1, 0,          # [6], 18, arg=1
+      _PY2_OPMAP["SETUP_LOOP"], 54, 0,         # [7], 21, dest=78
+      _PY2_OPMAP["LOAD_NAME"], 0, 0,           # [8], 24, arg=0
+      _PY2_OPMAP["GET_ITER"],                  # [9], 27
+      _PY2_OPMAP["FOR_ITER"], 46, 0,           # [10], 28, dest=77
+      _PY2_OPMAP["STORE_NAME"], 2, 0,          # [11], 31, arg=2
+      _PY2_OPMAP["SETUP_LOOP"], 37, 0,         # [12], 34, dest=74
+      _PY2_OPMAP["LOAD_NAME"], 0, 0,           # [13], 37, arg=0
+      _PY2_OPMAP["GET_ITER"],                  # [14], 40
+      _PY2_OPMAP["FOR_ITER"], 29, 0,           # [15], 41, dest=73
+      _PY2_OPMAP["STORE_NAME"], 3, 0,          # [16], 44, arg=3
+      _PY2_OPMAP["LOAD_NAME"], 2, 0,           # [17], 47, arg=2
+      _PY2_OPMAP["POP_JUMP_IF_FALSE"], 41, 0,  # [18], 50, dest=41
+      _PY2_OPMAP["LOAD_NAME"], 1, 0,           # [19], 53, arg=1
+      _PY2_OPMAP["LOAD_NAME"], 2, 0,           # [20], 56, arg=2
+      _PY2_OPMAP["LOAD_NAME"], 3, 0,           # [21], 59, arg=3
+      _PY2_OPMAP["BINARY_MULTIPLY"],           # [22], 62
+      _PY2_OPMAP["INPLACE_ADD"],               # [23], 63
+      _PY2_OPMAP["STORE_NAME"], 1, 0,          # [24], 64, arg=1
+      _PY2_OPMAP["JUMP_ABSOLUTE"], 41, 0,      # [25], 67, dest=41
+      _PY2_OPMAP["JUMP_ABSOLUTE"], 41, 0,      # [26], 70 (unreachable), dest=41
+      _PY2_OPMAP["POP_BLOCK"],                 # [27], 73
+      _PY2_OPMAP["JUMP_ABSOLUTE"], 28, 0,      # [28], 74, dest=28
+      _PY2_OPMAP["POP_BLOCK"],                 # [29], 77
+      _PY2_OPMAP["LOAD_CONST"], 4, 0,          # [30], 78, arg=4
+      _PY2_OPMAP["RETURN_VALUE"],              # [31], 81
   ])
 
   def test_each_instruction_once_loops(self):
@@ -153,17 +155,17 @@ class BytecodeTest(test_base.BaseTest, test_utils.MakeCodeMixin):
     x = 42
     """)
   code_deadcode = compat.int_array_to_bytes([
-      dis.opmap["LOAD_NAME"], 0, 0,           # [0] 0, arg=0
-      dis.opmap["POP_JUMP_IF_FALSE"], 15, 0,  # [1] 3, dest=15
-      dis.opmap["LOAD_CONST"], 0, 0,          # [2] 6, arg=0
-      dis.opmap["STORE_NAME"], 1, 0,          # [3] 9, arg=1
-      dis.opmap["JUMP_FORWARD"], 0, 0,        # [4] 12, dest=15
-      dis.opmap["LOAD_NAME"], 2, 0,           # [5] 15, arg=2
-      dis.opmap["RAISE_VARARGS"], 1, 0,       # [6] 18, arg=1
-      dis.opmap["LOAD_CONST"], 1, 0,          # [7] 21 (unreachable), arg=1
-      dis.opmap["STORE_NAME"], 1, 0,          # [8] 24 (unreachable), arg=1
-      dis.opmap["LOAD_CONST"], 2, 0,          # [9] 27 (unreachable), arg=2
-      dis.opmap["RETURN_VALUE"],              # [10] 30 (unreachable)
+      _PY2_OPMAP["LOAD_NAME"], 0, 0,           # [0] 0, arg=0
+      _PY2_OPMAP["POP_JUMP_IF_FALSE"], 15, 0,  # [1] 3, dest=15
+      _PY2_OPMAP["LOAD_CONST"], 0, 0,          # [2] 6, arg=0
+      _PY2_OPMAP["STORE_NAME"], 1, 0,          # [3] 9, arg=1
+      _PY2_OPMAP["JUMP_FORWARD"], 0, 0,        # [4] 12, dest=15
+      _PY2_OPMAP["LOAD_NAME"], 2, 0,           # [5] 15, arg=2
+      _PY2_OPMAP["RAISE_VARARGS"], 1, 0,       # [6] 18, arg=1
+      _PY2_OPMAP["LOAD_CONST"], 1, 0,          # [7] 21 (unreachable), arg=1
+      _PY2_OPMAP["STORE_NAME"], 1, 0,          # [8] 24 (unreachable), arg=1
+      _PY2_OPMAP["LOAD_CONST"], 2, 0,          # [9] 27 (unreachable), arg=2
+      _PY2_OPMAP["RETURN_VALUE"],              # [10] 30 (unreachable)
   ])
 
   def test_each_instruction_once_dead_code(self):
@@ -186,7 +188,7 @@ class TraceTest(test_base.BaseTest, test_utils.MakeCodeMixin):
   """Tests for opcode tracing in the VM."""
 
   def setUp(self):
-    super(TraceTest, self).setUp()
+    super().setUp()
     self.errorlog = errors.ErrorLog()
     self.trace_vm = TraceVM(self.options, self.loader)
 
@@ -228,7 +230,7 @@ class AnnotationsTest(test_base.BaseTest, test_utils.MakeCodeMixin):
   """Tests for recording annotations."""
 
   def setUp(self):
-    super(AnnotationsTest, self).setUp()
+    super().setUp()
     self.errorlog = errors.ErrorLog()
     self.vm = analyze.CallTracer(self.errorlog, self.options, self.loader)
 
@@ -242,7 +244,7 @@ class AnnotationsTest(test_base.BaseTest, test_utils.MakeCodeMixin):
 class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
 
   def setUp(self):
-    super(DirectorLineNumbersTest, self).setUp()
+    super().setUp()
     self.errorlog = errors.ErrorLog()
     self.vm = analyze.CallTracer(self.errorlog, self.options, self.loader)
 
@@ -258,10 +260,14 @@ class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
         )
       ]  # type: dict
     """)
-    # In Python 3.7, STORE_NAME v is on the `("hello",` line.
-    self.assertEqual({
-        3 if self.python_version >= (3, 7) else 4: "dict",
-    }, self.vm.director.type_comments)
+    # The line number of STORE_NAME v changes between versions.
+    if self.python_version >= (3, 8):
+      lineno = 2
+    elif self.python_version >= (3, 7):
+      lineno = 3
+    else:
+      lineno = 4
+    self.assertEqual({lineno: "dict"}, self.vm.director.type_comments)
 
   def test_type_comment_with_trailing_comma(self):
     self.run_program("""
@@ -276,11 +282,18 @@ class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
         ],  # some comment
       ]  # type: dict
     """)
-    # In Python 3.7, STORE_NAME v is on the `("hello",` line.
-    self.assertEqual({
-        3 if self.python_version >= (3, 7) else 4: "dict",
-        9: "dict",
-    }, self.vm.director.type_comments)
+    # The line numbers of STORE_NAME change between versions.
+    if self.python_version >= (3, 8):
+      v_lineno = 2
+      w_lineno = 7
+    elif self.python_version >= (3, 7):
+      v_lineno = 3
+      w_lineno = 9
+    else:
+      v_lineno = 4
+      w_lineno = 9
+    self.assertEqual({v_lineno: "dict", w_lineno: "dict"},
+                     self.vm.director.type_comments)
 
   def test_decorators(self):
     self.run_program("""
@@ -299,7 +312,14 @@ class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
         def bar():
           pass
     """)
-    self.assertEqual(self.vm.director.decorators, {6, 11})
+    if self.python_version >= (3, 8):
+      real_decorator_lineno = 7
+      decorator_lineno = 14
+    else:
+      real_decorator_lineno = 6
+      decorator_lineno = 11
+    self.assertEqual(
+        self.vm.director.decorators, {real_decorator_lineno, decorator_lineno})
 
   def test_stacked_decorators(self):
     self.run_program("""
@@ -312,7 +332,8 @@ class DirectorLineNumbersTest(test_base.BaseTest, test_utils.MakeCodeMixin):
       class A:
           pass
     """)
-    self.assertEqual(self.vm.director.decorators, {6})
+    lineno = 8 if self.python_version >= (3, 8) else 6
+    self.assertEqual(self.vm.director.decorators, {lineno})
 
   def test_overload(self):
     self.run_program("""
